/*
 * Copyright (c) 1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project. <http://java.apache.org/>"
 *
 * 4. The names "Java Apache Element Construction Set", "Java Apache ECS" and
 *    "Java Apache Project" must not be used to endorse or promote products 
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called 
 *    "Java Apache Element Construction Set" nor "Java Apache ECS" appear 
 *    in their names without prior written permission of the
 *    Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project. <http://java.apache.org/>"
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.ecs;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.IOException;

/**
    This class provides a common set of attributes set* methods for all classes.
    It is abstract to prevent direct instantiation.
    @version $Id: ElementAttributes.java,v 1.7 1999/04/23 00:55:41 jonbolt Exp $
    @author <a href="mailto:snagy@servletapi.com">Stephan Nagy</a>
    @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
*/

public abstract class ElementAttributes extends Element implements AttributeRegistry
{
    /** Filter to use to escape attribute input */
    private Filter attribute_filter = getFilter();   // By default the attribute filter and the element filter are the same.

    /** Should we filter the value of the element attributes */
    private boolean filter_attribute_state = false;

    /** Does this element attribute value need a =""? */
    public static final String NO_ATTRIBUTE_VALUE = "ECS_NO_ATTRIBUTE_VALUE";

    /** What the start modifier should be */
    private String start_modifier = new String();

    /** What the end modifier should be */
    private String end_modifier = new String();

    /** What the modifier should be...optimization variable */
    private boolean start_modifier_defined = false;

    /** What the modifier should be...optimization variable */
    private boolean end_modifier_defined = false;

    /**
        Basic constructor.  This class is abstract to prevent direct instantiation.
    */
    public ElementAttributes()
    {
    }
    
    /** Set the element id for Cascading Style Sheets. */
    public Element setID(String id)
    {
        addAttribute("id",id);
        return(this);
    }

    /** Set the element class for Cascading Style Sheets. */
    public Element setClass(String element_class)
    {
        addAttribute("class",element_class);
        return(this);
    }

    /**
        Sets the LANG="" attribute
        @param   lang  the LANG="" attribute
    */
    public Element setLang(String lang)
    {
        addAttribute("lang",lang);
        return(this);
    }

    /**
        Sets the STYLE="" attribute
        @param   style  the STYLE="" attribute
    */
    public Element setStyle(String style)
    {
        addAttribute("style",style);
        return(this);
    }
    /**
        Sets the DIR="" attribute
        @param   dir  the DIR="" attribute
    */
    public Element setDir(String dir)
    {
        addAttribute("dir",dir);
        return(this);
    }
    /**
        Sets the TITLE="" attribute
        @param   title  the TITLE="" attribute
    */
    public Element setTitle(String title)
    {
        addAttribute("title",title);
        return(this);
    }

    /**
        Find out if we want to filter the elements attributes or not.
    */
    protected boolean getAttributeFilterState()
    {
        return(filter_attribute_state);
    }
    
    /**
        Tell the element if we want to filter its attriubtes.
        @param filter_attribute_state do we want to filter the attributes of this element?
    */
    public Element setAttributeFilterState(boolean filter_attribute_state)
    {
        this.filter_attribute_state = filter_attribute_state;
        return(this);
    }

    /**
        Set up a new filter for all element attributes.
        @param Filter the filter we want to use for element attributes.  By <br>
        default it is the same as is used for the value of the tag. It is assumed<br>
        that if you create a new filter you must want to use it.
    */
    public Element setAttributeFilter(Filter attribute_filter)
    {
        filter_attribute_state = true; // If your setting up a filter you must want to filter.
        this.attribute_filter = attribute_filter;
        return(this);
    }

    /**
        Get the filter for all element attributes.
        @param Filter the filter we want to use for element attributes.  By <br>
        default it is the same as is used for the value of the tag. It is assumed<br>
        that if you create a new filter you must want to use it.
    */
    public Filter getAttributeFilter()
    {
        return(this.attribute_filter);
    }

    /**
        Set the element modifier the default is no modifier.
        @param modifier the modifier for the element a ? would result <br>
        in &lt;? ?&gt;
    */
    public Element setModifier(String modifier)
    {
        setStartModifier(modifier);
        setEndModifier(modifier);
        setStartModifierDefined(true);
        setEndModifierDefined(true);
        return(this);
    }

    /**
        Set the element modifier the default is no modifier.
        @param modifier the modifier for the element a ? would result <br>
        in &lt;? &gt;
    */
    public Element setStartModifier(String modifier)
    {
        this.start_modifier = modifier;
        setStartModifierDefined(true);
        return(this);
    }

    /**
        Set the element modifier the default is no modifier.
        @param modifier the modifier for the element a ? would result <br>
        in &lt; ?&gt;
    */
    public Element setEndModifier(String modifier)
    {
        this.end_modifier = modifier;
        setEndModifierDefined(true);
        return(this);
    }

    /**
        Set the element modifier the default is no modifier.
        @param modifier the modifier for the element a ? would result <br>
        in &lt;? ?&gt;
    */
    public Element setModifier(String start_modifier,String end_modifier)
    {
        this.start_modifier = start_modifier;
        this.end_modifier = end_modifier;
        setStartModifierDefined(true);
        setEndModifierDefined(true);
        return(this);
    }
    /**
        What is the element modifier for this tag.
    */
    public String getStartModifier()
    {
        return(start_modifier);
    }

    /**
        What is the element modifier for this tag.
    */
    public String getEndModifier()
    {
        return(end_modifier);
    }

    /**
        Remove the element modifier for this tag.
    */
    public Element removeStartModifier()
    {
        setStartModifierDefined(false);
        return(this);
    }

    /**
        Remove the element modifier for this tag.
    */
    public Element removeEndModifier()
    {
        setEndModifierDefined(false);
        return(this);
    }

    /**
        Modifier optimization
    */
    private void setStartModifierDefined(boolean val)
    {
        this.start_modifier_defined = val;
    }

    /**
        Modifier optimization
    */
    private void setEndModifierDefined(boolean val)
    {
        this.end_modifier_defined = val;
    }

    /**
        Modifier optimization
    */
    private boolean getStartModifierDefined()
    {
        return this.start_modifier_defined;
    }

    /**
        Modifier optimization
    */
    private boolean getEndModifierDefined()
    {
        return this.end_modifier_defined;
    }

    /** Add an attribute to the element. */
    public void addAttribute(String attribute_name, Object attribute_value)
    {
        getElementHashEntry().put(attribute_name, attribute_value);
    }

    public void removeAttribute(String attribute_name)
    {
        try
        {
            getElementHashEntry().remove(attribute_name);
        }
        catch ( Exception e )
        {
        }
    }

    public boolean hasAttribute(String attribute)
    {
        return(getElementHashEntry().containsKey(attribute));
    }
    /** Add the element to the designated OutputStream. */
    public void output(OutputStream out)
    {

        Hashtable ce = (Hashtable) getElementHashEntry().clone();
        String content = new String();

        try
        {
            String _tag = (String) ce.get("_tag");
            ce.remove("_tag"); // get rid of the tag
            if(ce.containsKey("_text"))
            {
                content = (String) ce.get("_text");
                ce.remove("_text"); // get rid of the _text attribute
            }
            out.write('<');

            if(getStartModifierDefined())
            {
                out.write(getStartModifier().getBytes());
            }

            out.write(alterCase(_tag).getBytes());
            Enumeration enum = ce.keys();
            String value = new String(); // avoid creating a new string object on each pass through the loop

            while (enum.hasMoreElements())
            {
                String attr = (String) enum.nextElement();
                if(getAttributeFilterState())
                {
                    value = getAttributeFilter().process((String) ce.get(attr));
                }
                else
                {
                    value = (String) ce.get(attr);
                }
                out.write(' ');
                out.write(alterCase(attr).getBytes());
                if(!value.equalsIgnoreCase(NO_ATTRIBUTE_VALUE))
                {
                    out.write('=');
                    out.write('"');
                    out.write(value.getBytes());
                    out.write('"');
                }
            }

            if(getEndModifierDefined())
            {
                out.write(getEndModifier().getBytes());
            }
            out.write('>');
            out.write(content.getBytes());

            // Not all multi-part elements need a closing tag make it optional
            if (getNeedClosingTag())
            {
                out.write('<');
                out.write('/');
                out.write(alterCase(_tag).getBytes());
                out.write('>');
            }

        }
        catch (IOException ioe)
        {
        }
    }

    /** Add element to the designated PrintWriter. */
    public void output(PrintWriter pw)
    {
        Hashtable ce = (Hashtable) getElementHashEntry().clone();
        String content = new String();

        String _tag = (String) ce.get("_tag");
        ce.remove("_tag"); // get rid of the tag
        if(ce.containsKey("_text"))
        {
            content = (String) ce.get("_text");
            ce.remove("_text"); // get rid of the _text attribute
        }

        pw.write('<');

        if(getStartModifierDefined())
        {
            pw.write(getStartModifier());
        }

        pw.write(alterCase(_tag));
        Enumeration enum = ce.keys();
        String value = new String();

        while (enum.hasMoreElements())
        {
            String attr = (String) enum.nextElement();
            if(getAttributeFilterState())
            {
                value = getAttributeFilter().process((String) ce.get(attr));
            }
            else
            {
                value = (String) ce.get(attr);
            }
            pw.write(' ');
            pw.write(alterCase(attr));
            if(!value.equalsIgnoreCase(NO_ATTRIBUTE_VALUE))
            {
                pw.write('=');
                pw.write('"');
                pw.write(value);
                pw.write('"');
            }
        }

        if(getEndModifierDefined())
        {
            pw.write(getEndModifier());
        }

        pw.write('>');
        pw.write(content);
        if(getNeedClosingTag())
        {
            pw.write('<');
            pw.write('/');
            pw.write(alterCase(_tag));
            pw.write('>');
        }
    }
}